/*
 * arith.h -- arithmetic routines
 */


#ifndef _ARITH_H_
#define _ARITH_H_


extern Octa zero;	/* 0 */
extern Octa negOne;	/* -1 */
extern Octa aux;	/* auxiliary output of a subroutine */
extern Bool overflow;	/* set by subroutines for signed arithmetic */
extern int exceptions;	/* bits set by floating point operations */
extern int curRound;	/* current rounding mode */


Octa oplus(Octa y, Octa z);	/* unsigned y + z */
Octa ominus(Octa y, Octa z);	/* unsigned y - z */
Octa incr(Octa y, int delta);	/* unsigned y + delta (delta is signed) */
Octa oand(Octa y, Octa z);	/* y & z */
Octa oandn(Octa y, Octa z);	/* y & ~z */
Octa oxor(Octa y, Octa z);	/* y ^ z */
Octa shiftLeft(Octa y, int s);	/* y << s, 0 <= s <= 64 */
Octa shiftRight(Octa y, int s, Bool u);	/* y >> s, signed if !u */
Octa omult(Octa y, Octa z);	/* unsigned (aux, x) = y * z */
Octa somult(Octa y, Octa z);	/* signed x = y * z, setting overflow */
Octa odiv(Octa x, Octa y, Octa z);	/* unsigned (x, y) / z, aux = rem */
Octa sodiv(Octa y, Octa z);	/* signed y / z, aux = rem */
int countBits(Tetra z);		/* population counter */
Tetra byteDiff(Tetra y, Tetra z);	/* half of BDIF */
Tetra wydeDiff(Tetra y, Tetra z);	/* half of WDIF */
Octa boolMult(Octa y, Octa z, Bool x);	/* MOR or MXOR */
Octa sfLoad(Tetra z);		/* load short float */
Tetra sfStore(Octa x);		/* store short float */
Octa fplus(Octa y, Octa z);	/* floating point x = y + z */
Octa fmult(Octa y, Octa z);	/* floating point x = y * z */
Octa fdivide(Octa y, Octa z);	/* floating point x = y / z */
Octa froot(Octa z, int mode);	/* floating point x = sqrt(z) */
Octa fremstep(Octa y, Octa z, int d);	/* floating point x rem z = y rem z */
Octa fintegerize(Octa z, int mode);	/* floating point x = round(z) */
int fcomp(Octa y, Octa z);	/* -1, 0, 1, or 2 if <, =, >, or unordered */
int fepscomp(Octa y, Octa z, Octa eps, int sim);	/* x = y ~ z(eps) */
Octa floatit(Octa z, int mode, int unsgnd, int shrt);	/* fix to float */
Octa fixit(Octa z, int mode);	/* float to fix */


#endif /* _ARITH_H_ */
