/*
 * cpu.h -- CPU simulation
 */


#ifndef _CPU_H_
#define _CPU_H_


#define TRIP_H_BIT	(1 << 16)	/* trip handler invocation */
#define TRIP_D_BIT	(1 << 15)	/* integer divide check */
#define TRIP_V_BIT	(1 << 14)	/* integer overflow */
#define TRIP_W_BIT	(1 << 13)	/* float-to-fix overflow */
#define TRIP_I_BIT	(1 << 12)	/* invalid operation */
#define TRIP_O_BIT	(1 << 11)	/* floating overflow */
#define TRIP_U_BIT	(1 << 10)	/* floating underflow */
#define TRIP_Z_BIT	(1 <<  9)	/* floating division by zero */
#define TRIP_X_BIT	(1 <<  8)	/* floating inexact */

#define IRQ_TERM_3_RCVR	48		/* terminal 3 receiver interrupt */
#define IRQ_TERM_3_XMTR	47		/* terminal 3 transmitter interrupt */
#define IRQ_TERM_2_RCVR	46		/* terminal 2 receiver interrupt */
#define IRQ_TERM_2_XMTR	45		/* terminal 2 transmitter interrupt */
#define IRQ_TERM_1_RCVR	44		/* terminal 1 receiver interrupt */
#define IRQ_TERM_1_XMTR	43		/* terminal 1 transmitter interrupt */
#define IRQ_TERM_0_RCVR	42		/* terminal 0 receiver interrupt */
#define IRQ_TERM_0_XMTR	41		/* terminal 0 transmitter interrupt */

#define IRQ_DISK	29		/* disk interrupt */


Octa cpuGetIP(void);
void cpuSetIP(Octa addr);

Octa cpuGetGR(int regnum);
void cpuSetGR(int regnum, Octa value);
Octa cpuGetLR(int regnum);
void cpuSetLR(int regnum, Octa value);
Octa cpuGetDR(int regnum);
void cpuSetDR(int regnum, Octa value);

Bool cpuTestBreak(void);
Octa cpuGetBreak(void);
void cpuSetBreak(Octa addr);
void cpuResetBreak(void);

void cpuStep(void);
void cpuRun(void);
void cpuHalt(void);

void cpuSetTrip(Tetra tripBitMask);
void cpuSetTrap(int trapBitNumber);

void cpuReset(void);
void cpuInit(void);
void cpuExit(void);


#endif /* _CPU_H_ */
